@@ -1,6 +1,6 @@
 /*
  *	aegis - project change supervisor
- *	Copyright (C) 1994 Peter Miller;
+ *	Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2001 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,25 +15,27 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: functions to implement the 'aegis -Delta_Name' command
  */
 
-#include <stdio.h>
+#include <ac/stdio.h>
 
 #include <aedn.h>
-#include <ael.h>
+#include <ael/project/history.h>
 #include <arglex2.h>
 #include <commit.h>
 #include <error.h>
 #include <help.h>
 #include <lock.h>
-#include <option.h>
+#include <progname.h>
 #include <project.h>
-#include <pstate.h>
-#include <user.h>
+#include <project_hist.h>
+#include <sub.h>
 #include <trace.h>
+#include <user.h>
+#include <zero.h>
 
 
 static void delta_name_usage _((void));
@@ -43,7 +45,7 @@ delta_name_usage()
 {
 	char		*progname;
 
-	progname = option_progname_get();
+	progname = progname_get();
 	fprintf
 	(
 		stderr,
@@ -66,12 +68,7 @@ static void delta_name_help _((void));
 static void
 delta_name_help()
 {
-	static char *text[] =
-	{
-#include <../man1/aedn.h>
-	};
-
-	help(text, SIZEOF(text), delta_name_usage);
+	help("aedn", delta_name_usage);
 }
 
 
@@ -97,22 +94,32 @@ delta_name_list()
 
 		case arglex_token_change:
 			if (arglex() != arglex_token_number)
-				delta_name_usage();
+				option_needs_number(arglex_token_change, delta_name_usage);
 			/* fall through... */
 
 		case arglex_token_number:
 			if (change_number)
-				fatal("duplicate -Change option");
+				duplicate_option_by_name(arglex_token_change, delta_name_usage);
 			change_number = arglex_value.alv_number;
-			if (change_number < 1)
-				fatal("change %ld out of range", change_number);
+			if (change_number == 0)
+				change_number = MAGIC_ZERO;
+			else if (change_number < 1)
+			{
+				sub_context_ty	*scp;
+
+				scp = sub_context_new();
+				sub_var_set_long(scp, "Number", change_number);
+				fatal_intl(scp, i18n("change $number out of range"));
+				/* NOTREACHED */
+				sub_context_delete(scp);
+			}
 			break;
 
 		case arglex_token_project:
 			if (arglex() != arglex_token_string)
-				delta_name_usage();
+				option_needs_name(arglex_token_project, delta_name_usage);
 			if (project_name)
-				fatal("duplicate -Project option");
+				duplicate_option_by_name(arglex_token_project, delta_name_usage);
 			project_name = str_from_c(arglex_value.alv_string);
 			break;
 		}
@@ -130,20 +137,16 @@ static void delta_name_main _((void));
 static void
 delta_name_main()
 {
+	sub_context_ty	*scp;
 	string_ty	*project_name;
 	long		delta_number;
 	string_ty	*delta_name;
 	int		stomp;
 	project_ty	*pp;
 	user_ty		*up;
-	pstate		pstate_data;
-	pstate_history_list h;
-	pstate_history	he;
-	string_ty	**addr_p;
-	type_ty		*type_p;
-	size_t		j;
 
 	trace(("delta_name_main()\n{\n"/*}*/));
+	arglex();
 	project_name = 0;
 	delta_number = 0;
 	delta_name = 0;
@@ -158,21 +161,14 @@ delta_name_main()
 
 		case arglex_token_overwriting:
 			if (stomp)
-			{
-				duplicate:
-				fatal
-				(
-					"duplicate %s option",
-					arglex_value.alv_string
-				);
-			}
+				duplicate_option(delta_name_usage);
 			stomp = 1;
 			break;
 
 		case arglex_token_string:
 			if (delta_name)
 			{
-				error("too many delta names specified");
+				error_intl(0, i18n("too many delta names"));
 				delta_name_usage();
 			}
 			delta_name = str_from_c(arglex_value.alv_string);
@@ -181,26 +177,40 @@ delta_name_main()
 		case arglex_token_number:
 			if (delta_number)
 			{
-				error("too many delta numbers specified");
+				error_intl(0, i18n("too many delta numbers"));
 				delta_name_usage();
 			}
 			delta_number = arglex_value.alv_number;
 			if (delta_number < 1)
-				fatal("delta %ld out of range", delta_number);
+			{
+				scp = sub_context_new();
+				sub_var_set_long(scp, "Number", delta_number);
+				fatal_intl(scp, i18n("delta $number out of range"));
+				/* NOTREACHED */
+				sub_context_delete(scp);
+			}
 			break;
 
 		case arglex_token_project:
 			if (project_name)
-				goto duplicate;
+				duplicate_option(delta_name_usage);
 			if (arglex() != arglex_token_string)
-				delta_name_usage();
+				option_needs_name(arglex_token_project, delta_name_usage);
 			project_name = str_from_c(arglex_value.alv_string);
 			break;
+
+		case arglex_token_wait:
+		case arglex_token_wait_not:
+			user_lock_wait_argument(delta_name_usage);
+			break;
 		}
 		arglex();
 	}
 	if (!delta_name)
-		fatal("no delta name specified");
+	{
+		error_intl(0, i18n("no delta name"));
+		delta_name_usage();
+	}
 
 	/*
 	 * locate project data
@@ -221,118 +231,58 @@ delta_name_main()
 	 */
 	project_pstate_lock_prepare(pp);
 	lock_take();
-	pstate_data = project_pstate_get(pp);
-	h = pstate_data->history;
 
 	/*
 	 * it is an error if the user is not a project administrator
 	 */
 	if (!project_administrator_query(pp, user_name(up)))
-	{
-		project_fatal
-		(
-			pp,
-			"user \"%S\" is not an administrator",
-			user_name(up)
-		);
-	}
+		project_fatal(pp, 0, i18n("not an administrator"));
 
 	/*
 	 * it is an error if the delta does not exist
 	 */
-	he = 0;
 	if (delta_number)
 	{
-		for (j = 0; j < h->length; ++j)
+		if (!project_history_delta_validate(pp, delta_number))
 		{
-			if (h->list[j]->delta_number == delta_number)
-			{
-				he = h->list[j];
-				break;
-			}
+			scp = sub_context_new();
+			sub_var_set_long(scp, "Number", delta_number);
+			project_fatal(pp, scp, i18n("no delta $number"));
+			/* NOTREACHED */
+			sub_context_delete(scp);
 		}
-		if (!he)
-			project_fatal(pp, "delta number %ld unknown", delta_number);
 	}
 	else
 	{
-		if (!h || !h->length)
-		{
-			project_fatal
-			(
-				pp,
-"no integrations have been completed, \
-so it is not yet possible to assign a delta name"
-			);
-		}
-		he = h->list[h->length - 1];
-		delta_number = he->delta_number;
+		delta_number = project_history_delta_latest(pp);
+		if (delta_number <= 0)
+			project_fatal(pp, 0, i18n("no delta yet"));
 	}
 
-	/*
-	 * make sure the name has not been used already
-	 */
-	for (j = 0; j < h->length; ++j)
+	if (!stomp)
 	{
-		pstate_history	he2;
-		pstate_history_name_list nlp;
-		size_t		k, m;
+		long		other;
 
-		he2 = h->list[j];
-		nlp = he2->name;
-		if (!nlp || !nlp->length)
-			continue;
-		for (k = 0; k < nlp->length; ++k)
+		other = project_history_delta_by_name(pp, delta_name, 1);
+		if (other && other != delta_number)
 		{
-			/*
-			 * see if the name matches
-			 */
-			if (!str_equal(nlp->list[k], delta_name))
-				continue;
-
-			/*
-			 * it is an error if the name has been used and
-			 * the -OverWriting option was not given
-			 */
-			if (!stomp && he2->delta_number != delta_number)
-			{
-				project_fatal
-				(
-					pp,
-"the delta name \"%S\" is already assigned to delta number %ld; \
-to reassign this name to delta number %ld \
-you must use the -OverWriting option",
-					delta_name,
-					he2->delta_number,
-					delta_number
-				);
-			}
-
-			/*
-			 * remove the name from the list
-			 */
-			str_free(nlp->list[k]);
-			for (m = k + 1; m < nlp->length; ++m)
-				nlp->list[m - 1] = nlp->list[m];
-			nlp->length--;
-			if (nlp->length == 0)
-			{
-				he2->name = 0;
-				pstate_history_name_list_type.free(nlp);
-				break;
-			}
-			k--;
+			scp = sub_context_new();
+			sub_var_set_string(scp, "Name", delta_name);
+			sub_var_set_long(scp, "Number", delta_number);
+			sub_var_optional(scp, "Number");
+			sub_var_set_long(scp, "Other", other);
+			sub_var_optional(scp, "Other");
+			project_fatal(pp, scp, i18n("delta $name in use"));
+			/* NOTREACHED */
+			sub_context_delete(scp);
 		}
 	}
+	project_history_delta_name_delete(pp, delta_name);
 
 	/*
 	 * add the name to the selected history entry
 	 */
-	if (!he->name)
-		he->name = pstate_history_name_list_type.alloc();
-	addr_p = pstate_history_name_list_type.list_parse(he->name, &type_p);
-	assert(type_p == &string_type);
-	*addr_p = str_copy(delta_name);
+	project_history_delta_name_add(pp, delta_number, delta_name);
 
 	/*
 	 * release the locks
@@ -344,13 +294,13 @@ you must use the -OverWriting option",
 	/*
 	 * verbose success message
 	 */
-	project_verbose
-	(
-		pp,
-		"delta %ld may now be refered to as delta \"%S\"",
-		delta_number,
-		delta_name
-	);
+	scp = sub_context_new();
+	sub_var_set_string(scp, "Name", delta_name);
+	sub_var_optional(scp, "Name");
+	sub_var_set_long(scp, "Number", delta_number);
+	sub_var_optional(scp, "Number");
+	project_verbose(pp, scp, i18n("delta name complete"));
+	sub_context_delete(scp);
 	project_free(pp);
 	user_free(up);
 	str_free(delta_name);
@@ -361,20 +311,13 @@ you must use the -OverWriting option",
 void
 delta_name_assignment()
 {
-	trace(("delta_name()\n{\n"/*}*/));
-	switch (arglex())
+	static arglex_dispatch_ty dispatch[] =
 	{
-	default:
-		delta_name_main();
-		break;
+		{ arglex_token_help,		delta_name_help,	},
+		{ arglex_token_list,		delta_name_list,	},
+	};
 
-	case arglex_token_help:
-		delta_name_help();
-		break;
-
-	case arglex_token_list:
-		delta_name_list();
-		break;
-	}
-	trace((/*{*/"}\n"));
+	trace(("delta_name_assignment()\n{\n"));
+	arglex_dispatch(dispatch, SIZEOF(dispatch), delta_name_main);
+	trace(("}\n"));
 }
