@@ -1,6 +1,6 @@
 /*
  *	aegis - project change supervisor
- *	Copyright (C) 1991, 1992, 1993, 1994, 1995 Peter Miller;
+ *	Copyright (C) 1991-2001 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,33 +15,39 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: functions for implementing integrate begin
  */
 
-#include <stdio.h>
+#include <ac/stdio.h>
 #include <ac/stdlib.h>
 #include <ac/string.h>
 #include <ac/time.h>
+#include <ac/unistd.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
 
 #include <aeib.h>
-#include <ael.h>
+#include <ael/change/by_state.h>
 #include <arglex2.h>
 #include <commit.h>
 #include <change.h>
+#include <change/branch.h>
+#include <change/file.h>
 #include <dir.h>
 #include <error.h>
 #include <file.h>
+#include <gmatch.h>
 #include <help.h>
 #include <lock.h>
 #include <log.h>
-#include <option.h>
+#include <progname.h>
 #include <os.h>
 #include <project.h>
+#include <project/file.h>
+#include <project_hist.h>
 #include <sub.h>
 #include <trace.h>
 #include <undo.h>
@@ -55,7 +61,7 @@ integrate_begin_usage()
 {
 	char		*progname;
 
-	progname = option_progname_get();
+	progname = progname_get();
 	fprintf(stderr, "usage: %s -Integrate_Begin [ <option>... ]\n", progname);
 	fprintf(stderr, "       %s -Integrate_Begin -List [ <option>... ]\n", progname);
 	fprintf(stderr, "       %s -Integrate_Begin -Help\n", progname);
@@ -68,12 +74,7 @@ static void integrate_begin_help _((void));
 static void
 integrate_begin_help()
 {
-	static char *text[] =
-	{
-#include <../man1/aeib.h>
-	};
-
-	help(text, SIZEOF(text), integrate_begin_usage);
+	help("aeib", integrate_begin_usage);
 }
 
 
@@ -97,9 +98,9 @@ integrate_begin_list()
 
 		case arglex_token_project:
 			if (arglex() != arglex_token_string)
-				integrate_begin_usage();
+				option_needs_name(arglex_token_project, integrate_begin_usage);
 			if (project_name)
-				fatal("duplicate -Project option");
+				duplicate_option_by_name(arglex_token_project, integrate_begin_usage);
 			project_name = str_from_c(arglex_value.alv_string);
 			break;
 		}
@@ -116,6 +117,86 @@ integrate_begin_list()
 }
 
 
+static string_ty *remove_comma_d_if_present _((string_ty *));
+
+static string_ty *
+remove_comma_d_if_present(s)
+	string_ty	*s;
+{
+	char		*cp;
+
+	if (s->str_length < 2)
+		return str_copy(s);
+	cp = s->str_text + s->str_length - 2;
+	if (cp[0] == ',' && cp[1] == 'D')
+		return str_n_from_c(s->str_text, s->str_length - 2);
+	return str_copy(s);
+}
+
+
+static int isa_suppressed_filename _((change_ty *, string_ty *));
+
+static int
+isa_suppressed_filename(cp, fn)
+	change_ty	*cp;
+	string_ty	*fn;
+{
+	pconf		pconf_data;
+	pconf_integrate_begin_exceptions_list p;
+	size_t		j;
+
+	pconf_data = change_pconf_get(cp, 1);
+	p = pconf_data->integrate_begin_exceptions;
+	if (!p)
+		return 0;
+	for (j = 0; j < p->length; ++j)
+	{
+		if (gmatch(p->list[j]->str_text, fn->str_text))
+			return 1;
+	}
+	return 0;
+}
+
+
+static void chmod_common _((string_ty *, struct stat *, int, change_ty *));
+
+static void
+chmod_common(filename, st, rdwr, cp)
+	string_ty	*filename;
+	struct stat	*st;
+	int		rdwr;
+	change_ty	*cp;
+{
+	int		mode;
+
+	if (rdwr)
+	{
+		/*
+		 * If it is not a source file, it could be owned
+		 * by some other user, and we have no control
+		 * over its owner or mode.  Report a warning if
+		 * we can't change the mode.
+		 *
+		 * Also, we leave it writable if it is already.
+		 * This is normal for generated files.
+		 */
+		mode = (st->st_mode | 0644) & ~0022;
+	}
+	else
+	{
+		/*
+		 * Source files, on the other hand, should always
+		 * be owned by us, and thus always chmod(2)able
+		 * by us.  Have a hissy-fit if they aren't.
+		 * 
+		 * Also, source files should be read-only.
+		 */
+		mode = (st->st_mode | 0444) & ~0222;
+	}
+	os_chmod(filename, mode);
+}
+
+
 static void link_tree_callback_minimum _((void *, dir_walk_message_ty,
 	string_ty *, struct stat *));
 
@@ -127,12 +208,16 @@ link_tree_callback_minimum(arg, message, path, st)
 	struct stat	*st;
 {
 	string_ty	*s1;
+	string_ty	*s1short;
 	string_ty	*s2;
 	change_ty	*cp;
-	pstate_src	src;
+	fstate_src	src;
+	int		exists;
+	int		remove_the_file;
 
 	trace(("link_tree_callback_minimum(message = %d, path = %08lX, \
 st = %08lX)\n{\n"/*}*/, message, path, st));
+	os_interrupt_cope();
 	cp = (change_ty *)arg;
 	assert(cp);
 	trace_string(path->str_text);
@@ -151,7 +236,6 @@ st = %08lX)\n{\n"/*}*/, message, path, st));
 		{
 			assert(!os_exists(s2));
 			os_mkdir(s2, 02755);
-			undo_rmdir_errok(s2);
 		}
 		break;
 
@@ -162,23 +246,41 @@ st = %08lX)\n{\n"/*}*/, message, path, st));
 			 * Don't link files with horrible modes.
 			 * They shouldn't be source, anyway.
 			 */
-			if (!project_src_find(cp->pp, s1))
+			project_become_undo();
+			exists = 0 != project_file_find(cp->pp, s1);
+			project_become(cp->pp);
+			if (!exists)
 				break;
 		}
 
 		/*
-		 * don't link it if it's not a source file
+		 * Remove the ,D suffix, if present, and use the
+		 * shortened from to test for project and change
+		 * membership.  This ensures that diff files are also
+		 * copied across.  Note that deleted files *have*
+		 * difference files.
 		 */
-		src = project_src_find(cp->pp, s1);
+		s1short = remove_comma_d_if_present(s1);
+
+		/*
+		 * Don't link it if it's not a source file, or a
+		 * relevant ,D file.
+		 */
+		project_become_undo();
+		src = project_file_find(cp->pp, s1short);
+		project_become(cp->pp);
 		if
 		(
 			!src
 		||
-			src->deleted_by
+			(src->deleted_by && str_equal(s1, s1short))
 		||
 			src->about_to_be_created_by
 		)
+		{
+			str_free(s1short);
 			break;
+		}
 
 		/*
 		 * make sure the directory is there
@@ -191,17 +293,51 @@ st = %08lX)\n{\n"/*}*/, message, path, st));
 		);
 
 		/*
-		 * don't link file if in change
+		 * Don't link a file (or the corresponding ,D file) if
+		 * the file is in the change.
 		 */
-		if (change_src_find(cp, s1))
+		if (change_file_find(cp, s1short))
+		{
+			str_free(s1short);
 			break;
+		}
+
+		/*
+		 * Don't link a suppressed file.
+		 * BUT keep primary source files and their diff files.
+		 */
+		project_become_undo();
+		remove_the_file =
+			(
+				!project_file_find(cp->pp, s1short)
+			&&
+				isa_suppressed_filename(cp, s1short)
+			);
+		project_become(cp->pp);
+		if (remove_the_file)
+		{
+			str_free(s1short);
+			break;
+		}
+		str_free(s1short);
 
 		/*
 		 * link the file and make sure it is a suitable mode
 		 */
+		trace(("ln %s %s\n", path->str_text, s2->str_text));
 		os_link(path, s2);
-		undo_unlink_errok(s2);
-		os_chmod(s2, (st->st_mode | 0444) & ~0222 & ~change_umask(cp));
+		project_become_undo();
+		exists = 0 != project_file_find(cp->pp, s1);
+		project_become(cp->pp);
+		chmod_common(s2, st, !exists, cp);
+
+		/*
+		 * Update the modify time of the linked file.  On a
+		 * fully-functional Unix, this is unnecessary, because a
+		 * hard link alters the ctime, not the mtime, and this
+		 * is a no-op.  Solaris, on the other hand, is brain dead.
+		 */
+		os_mtime_set(s2, st->st_mtime);
 		break;
 
 	case dir_walk_dir_after:
@@ -212,9 +348,9 @@ st = %08lX)\n{\n"/*}*/, message, path, st));
 		/*
 		 * ignore special files
 		 *
-		 * They could never be source files,
-		 * so they must be created by the build.
-		 * These ones must always be created at build time, that's all.
+		 * They could never be source files, so they must be
+		 * created by the build.  These ones must always be
+		 * created at build time, that's all.
 		 */
 		break;
 	}
@@ -235,12 +371,16 @@ link_tree_callback(arg, message, path, st)
 	struct stat	*st;
 {
 	string_ty	*s1;
+	string_ty	*s1short;
 	string_ty	*s2;
 	change_ty	*cp;
-	pstate_src	src;
+	fstate_src	src;
+	string_ty	*contents;
+	int		remove_the_file;
 
 	trace(("link_tree_callback(message = %d, path = %08lX, \
 st = %08lX)\n{\n"/*}*/, message, path, st));
+	os_interrupt_cope();
 	cp = (change_ty *)arg;
 	assert(cp);
 	trace_string(path->str_text);
@@ -257,11 +397,12 @@ st = %08lX)\n{\n"/*}*/, message, path, st));
 	case dir_walk_dir_before:
 		assert(!os_exists(s2));
 		os_mkdir(s2, 02755);
-		undo_rmdir_errok(s2);
 		break;
 
 	case dir_walk_file:
-		src = project_src_find(cp->pp, s1);
+		project_become_undo();
+		src = project_file_find(cp->pp, s1);
+		project_become(cp->pp);
 		if (st->st_mode & 07000)
 		{
 			/*
@@ -273,33 +414,66 @@ st = %08lX)\n{\n"/*}*/, message, path, st));
 		}
 
 		/*
-		 * don't link change files
+		 * Don't link a file (or the corresponding ,D file) if
+		 * the file is in the change.
 		 */
-		if (change_src_find(cp, s1))
+		s1short = remove_comma_d_if_present(s1);
+		if (change_file_find(cp, s1short))
+		{
+			str_free(s1short);
 			break;
+		}
+
+		/*
+		 * Don't link a suppressed file.
+		 * BUT keep primary source files and their diff files.
+		 */
+		project_become_undo();
+		remove_the_file =
+			(
+				!project_file_find(cp->pp, s1short)
+			&&
+				isa_suppressed_filename(cp, s1short)
+			);
+		project_become(cp->pp);
+		if (remove_the_file)
+		{
+			str_free(s1short);
+			break;
+		}
+		str_free(s1short);
 
 		/*
 		 * link the file and make sure it is a suitable mode
 		 */
 		os_link(path, s2);
-		undo_unlink_errok(s2);
-		if (!src)
-			os_chmod(s2, (st->st_mode | 0644) & ~0022 & ~change_umask(cp));
-		else
-			os_chmod(s2, (st->st_mode | 0444) & ~0222 & ~change_umask(cp));
+		chmod_common(s2, st, !src, cp);
+
+		/*
+		 * Update the modify time of the linked file.  On a
+		 * fully-functional Unix, this is unnecessary, because a
+		 * hard link alters the ctime, not the mtime, and this
+		 * is a no-op.  Solaris, on the other hand, is brain dead.
+		 */
+		os_mtime_set(s2, st->st_mtime);
 		break;
 
 	case dir_walk_dir_after:
 		break;
 
-	case dir_walk_special:
 	case dir_walk_symlink:
+		contents = os_readlink(path);
+		os_symlink(contents, s2);
+		str_free(contents);
+		break;
+
+	case dir_walk_special:
 		/*
 		 * ignore special files
 		 *
-		 * They could never be source files,
-		 * so they must be created by the build.
-		 * These ones must always be created at build time, that's all.
+		 * They could never be source files, so they must be
+		 * created by the build.  These ones must always be
+		 * created at build time, that's all.
 		 */
 		break;
 	}
@@ -320,13 +494,17 @@ copy_tree_callback_minimum(arg, message, path, st)
 	struct stat	*st;
 {
 	string_ty	*s1;
+	string_ty	*s1short;
 	string_ty	*s2;
 	change_ty	*cp;
-	pstate_src	src;
+	fstate_src	src;
 	int		uid;
+	int		exists;
+	int		remove_the_file;
 
-	trace(("copy_tree_callback(message = %d, path = %08lX, \
+	trace(("copy_tree_callback_minimum(message = %d, path = %08lX, \
 st = %08lX)\n{\n"/*}*/, message, path, st));
+	os_interrupt_cope();
 	cp = (change_ty *)arg;
 	assert(cp);
 	trace_string(path->str_text);
@@ -353,42 +531,55 @@ st = %08lX)\n{\n"/*}*/, message, path, st));
 		{
 			assert(!os_exists(s2));
 			os_mkdir(s2, 02755);
-			undo_rmdir_errok(s2);
 		}
 		break;
 
 	case dir_walk_file:
-		if (st->st_mode & 07000)
+		/*
+		 * Don't copy files which don't belong to us.
+		 * Don't copy files with horrible modes.
+		 * They shouldn't be source, anyway.
+		 */
+		os_become_query(&uid, (int *)0, (int *)0);
+		if ((st->st_uid != uid) || (st->st_mode & 07000))
 		{
-			/*
-			 * Don't copy files with horrible modes.
-			 * They shouldn't be source, anyway.
-			 */
-			if (!project_src_find(cp->pp, s1))
+			project_become_undo();
+			exists = 0 != project_file_find(cp->pp, s1);
+			project_become(cp->pp);
+			if (!exists)
 				break;
 		}
 
 		/*
-		 * Don't copy files which don't belong to us.
-		 * They shouldn't be source, anyway.
+		 * Remove the ,D suffix, if present, and use the
+		 * shortened from to test for project and change
+		 * membership.  This ensures that diff files are also
+		 * copied across.  Note that deleted files *have*
+		 * difference files.
 		 */
-		os_become_query(&uid, (int *)0, (int *)0);
-		src = project_src_find(cp->pp, s1);
-		if (st->st_uid != uid && !src)
-			break;
+		s1short = remove_comma_d_if_present(s1);
+		trace_string(s1short->str_text);
 
 		/*
-		 * don't copy it if it's not a source file
+		 * Don't copy it if it's not a source file, or a
+		 * relevant ,D file.
 		 */
-		if
-		(
-			!src
-		||
-			src->deleted_by
-		||
-			src->about_to_be_created_by
-		)
+		project_become_undo();
+		src = project_file_find(cp->pp, s1short);
+		remove_the_file =
+			(
+				!src
+			||
+				(src->deleted_by && str_equal(s1, s1short))
+			||
+				src->about_to_be_created_by
+			);
+		project_become(cp->pp);
+		if (remove_the_file)
+		{
+			str_free(s1short);
 			break;
+		}
 
 		/*
 		 * make sure the directory is there
@@ -401,17 +592,43 @@ st = %08lX)\n{\n"/*}*/, message, path, st));
 		);
 
 		/*
-		 * don't copy change files
+		 * Don't copy a file (or the corresponding ,D file) if
+		 * the file is in the change.
 		 */
-		if (change_src_find(cp, s1))
+		if (change_file_find(cp, s1short))
+		{
+			str_free(s1short);
 			break;
+		}
+
+		/*
+		 * Don't copy a suppressed file.
+		 * BUT keep primary source files and their diff files.
+		 */
+		project_become_undo();
+		remove_the_file =
+			(
+				!project_file_find(cp->pp, s1short)
+			&&
+				isa_suppressed_filename(cp, s1short)
+			);
+		project_become(cp->pp);
+		if (remove_the_file)
+		{
+			str_free(s1short);
+			break;
+		}
+		str_free(s1short);
 
 		/*
 		 * copy the file
 		 */
-		copy_whole_file(path, s2, 0);
-		undo_unlink_errok(s2);
-		os_chmod(s2, (st->st_mode | 0444) & ~0222 & ~change_umask(cp));
+		trace(("cp %s %s\n", path->str_text, s2->str_text));
+		copy_whole_file(path, s2, 1);
+		project_become_undo();
+		exists = 0 != project_file_find(cp->pp, s1);
+		project_become(cp->pp);
+		chmod_common(s2, st, !exists, cp);
 		break;
 
 	case dir_walk_dir_after:
@@ -422,9 +639,9 @@ st = %08lX)\n{\n"/*}*/, message, path, st));
 		/*
 		 * ignore special files
 		 *
-		 * They could never be source files,
-		 * so they must be created by the build.
-		 * These ones must always be created at build time, that's all.
+		 * They could never be source files, so they must be
+		 * created by the build.  These ones must always be
+		 * created at build time, that's all.
 		 */
 		break;
 	}
@@ -445,12 +662,16 @@ copy_tree_callback(arg, message, path, st)
 	struct stat	*st;
 {
 	string_ty	*s1;
+	string_ty	*s1short;
 	string_ty	*s2;
 	change_ty	*cp;
-	pstate_src	src;
 	int		uid;
+	string_ty	*contents;
+	int		exists;
+	int		remove_the_file;
 
 	trace(("copy_tree_callback(message = %d, path = %08lX, st = %08lX)\n{\n"/*}*/, message, path, st));
+	os_interrupt_cope();
 	cp = (change_ty *)arg;
 	assert(cp);
 	trace_string(path->str_text);
@@ -475,54 +696,89 @@ copy_tree_callback(arg, message, path, st)
 	case dir_walk_dir_before:
 		assert(!os_exists(s2));
 		os_mkdir(s2, 02755);
-		undo_rmdir_errok(s2);
 		break;
 
 	case dir_walk_file:
 		/*
+		 * Don't copy files which don't belong to us.
 		 * Don't copy files with horrible modes.
 		 * They shouldn't be source, anyway.
 		 */
-		src = project_src_find(cp->pp, s1);
-		if ((st->st_mode & 07000) && !src)
-			break;
-
-		/*
-		 * Don't copy files which don't belong to us.
-		 * They shouldn't be source, anyway.
-		 */
 		os_become_query(&uid, (int *)0, (int *)0);
-		if (st->st_uid != uid && !src)
-			break;
+		if ((st->st_uid != uid) || (st->st_mode & 07000))
+		{
+			project_become_undo();
+			exists = 0 != project_file_find(cp->pp, s1);
+			project_become(cp->pp);
+			if (!exists)
+				break;
+		}
 
 		/*
-		 * don't copy change files
+		 * Remove the ,D suffix, if present, and use the
+		 * shortened from to test for project and change
+		 * membership.  This ensures that diff files are also
+		 * copied across.  Note that deleted files *have*
+		 * difference files.
 		 */
-		if (change_src_find(cp, s1))
+		s1short = remove_comma_d_if_present(s1);
+		trace_string(s1short->str_text);
+
+		/*
+		 * Don't copy a file (or the corresponding ,D file) if
+		 * the file is in the change.
+		 */
+		if (change_file_find(cp, s1short))
+		{
+			str_free(s1short);
 			break;
+		}
+
+		/*
+		 * Don't copy a suppressed file.
+		 * BUT keep primary source files and their diff files.
+		 */
+		project_become_undo();
+		remove_the_file =
+			(
+				!project_file_find(cp->pp, s1short)
+			&&
+				isa_suppressed_filename(cp, s1short)
+			);
+		project_become(cp->pp);
+		if (remove_the_file)
+		{
+			str_free(s1short);
+			break;
+		}
+		str_free(s1short);
 
 		/*
 		 * copy the file
 		 */
-		copy_whole_file(path, s2, 0);
-		undo_unlink_errok(s2);
-		if (!src)
-			os_chmod(s2, (st->st_mode | 0644) & ~0022 & ~change_umask(cp));
-		else
-			os_chmod(s2, (st->st_mode | 0444) & ~0222 & ~change_umask(cp));
+		copy_whole_file(path, s2, 1);
+		project_become_undo();
+		exists = 0 != project_file_find(cp->pp, s1);
+		project_become(cp->pp);
+		chmod_common(s2, st, !exists, cp);
 		break;
 
 	case dir_walk_dir_after:
 		break;
 
-	case dir_walk_special:
 	case dir_walk_symlink:
+		contents = os_readlink(path);
+		os_symlink(contents, s2);
+		str_free(contents);
+		break;
+
+	case dir_walk_special:
 		/*
 		 * ignore special files
 		 *
-		 * They could never be source files,
-		 * so they must be created by the build.
-		 * These ones must always be created at build time, that's all.
+		 * They could never be source files, so they must be
+		 * created by the build.  These ones must always be
+		 * created at build time, that's all.
 		 */
 		break;
 	}
@@ -532,40 +788,6 @@ copy_tree_callback(arg, message, path, st)
 }
 
 
-static void insert_this_year _((pstate));
-
-static void
-insert_this_year(pstate_data)
-	pstate		pstate_data;
-{
-	time_t		now;
-	struct tm	*tm;
-	int		year;
-	size_t		j;
-	long		*year_p;
-	type_ty		*type_p;
-
-	time(&now);
-	tm = localtime(&now);
-	year = 1900 + tm->tm_year;
-
-	if (!pstate_data->copyright_years)
-		pstate_data->copyright_years =
-			pstate_copyright_years_list_type.alloc();
-	for (j = 0; j < pstate_data->copyright_years->length; ++j)
-		if (pstate_data->copyright_years->list[j] == year)
-			return;
-	year_p =
-		pstate_copyright_years_list_type.list_parse
-		(
-			pstate_data->copyright_years,
-			&type_p
-		);
-	assert(type_p == &integer_type);
-	*year_p = year;
-}
-
-
 static void integrate_begin_main _((void));
 
 static void
@@ -575,23 +797,32 @@ integrate_begin_main()
 	string_ty	*dd;
 	string_ty	*id;
 	pconf		pconf_data;
-	pstate		pstate_data;
 	cstate		cstate_data;
 	int		j;
 	cstate_history	history_data;
 	int		minimum;
+	int		maximum;
 	string_ty	*project_name;
 	project_ty	*pp;
 	long		change_number;
 	change_ty	*cp;
 	user_ty		*up;
+	user_ty		*pup;
 	int		errs;
 	string_ty	*s;
+	long		other;
+	log_style_ty	log_style;
+	string_ty	*base;
+	int		base_max;
+	string_ty	*num;
 
 	trace(("integrate_begin_main()\n{\n"/*}*/));
+	arglex();
 	minimum = 0;
+	maximum = 0;
 	project_name = 0;
 	change_number = 0;
+	log_style = log_style_create_default;
 	while (arglex_token != arglex_token_eoln)
 	{
 		switch (arglex_token)
@@ -602,39 +833,60 @@ integrate_begin_main()
 
 		case arglex_token_change:
 			if (arglex() != arglex_token_number)
-				integrate_begin_usage();
+				option_needs_number(arglex_token_change, integrate_begin_usage);
 			/* fall through... */
 
 		case arglex_token_number:
 			if (change_number)
-				fatal("duplicate -Change option");
+				duplicate_option_by_name(arglex_token_change, integrate_begin_usage);
 			change_number = arglex_value.alv_number;
-			if (change_number < 1)
-				fatal("change %ld out of range", change_number);
+			if (change_number == 0)
+				change_number = MAGIC_ZERO;
+			else if (change_number < 1)
+			{
+				sub_context_ty	*scp;
+
+				scp = sub_context_new();
+				sub_var_set_long(scp, "Number", change_number);
+				fatal_intl(scp, i18n("change $number out of range"));
+				/* NOTREACHED */
+				sub_context_delete(scp);
+			}
 			break;
 
 		case arglex_token_minimum:
 			if (minimum)
-			{
-				fatal
-				(
-					"duplicate %s option",
-					arglex_value.alv_string
-				);
-			}
+				duplicate_option(integrate_begin_usage);
 			minimum = 1;
 			break;
 
+		case arglex_token_maximum:
+			if (maximum)
+				duplicate_option(integrate_begin_usage);
+			maximum = 1;
+			break;
+
 		case arglex_token_project:
 			if (arglex() != arglex_token_string)
-				integrate_begin_usage();
+				option_needs_name(arglex_token_project, integrate_begin_usage);
 			/* fall through... */
 
 		case arglex_token_string:
 			if (project_name)
-				fatal("duplicate -Project option");
+				duplicate_option_by_name(arglex_token_project, integrate_begin_usage);
 			project_name = str_from_c(arglex_value.alv_string);
 			break;
+
+		case arglex_token_nolog:
+			if (log_style == log_style_none)
+				duplicate_option(integrate_begin_usage);
+			log_style = log_style_none;
+			break;
+
+		case arglex_token_wait:
+		case arglex_token_wait_not:
+			user_lock_wait_argument(integrate_begin_usage);
+			break;
 		}
 		arglex();
 	}
@@ -658,6 +910,7 @@ integrate_begin_main()
 	 */
 	if (!change_number)
 		change_number = user_default_change(up);
+	trace_long(change_number);
 	cp = change_alloc(pp, change_number);
 	change_bind_existing(cp);
 
@@ -668,7 +921,6 @@ integrate_begin_main()
 	change_cstate_lock_prepare(cp);
 	user_ustate_lock_prepare(up);
 	lock_take();
-	pstate_data = project_pstate_get(pp);
 	cstate_data = change_cstate_get(cp);
 	pconf_data = change_pconf_get(cp, 1);
 
@@ -676,84 +928,114 @@ integrate_begin_main()
 	 * make sure they are allowed to
 	 */
 	if (!project_integrator_query(pp, user_name(up)))
-	{
-		project_fatal
-		(
-			pp,
-			"user \"%S\" is not an integrator",
-			user_name(up)
-		);
-	}
+		project_fatal(pp, 0, i18n("not an integrator"));
 	if (cstate_data->state != cstate_state_awaiting_integration)
-	{
-		change_fatal
-		(
-			cp,
-"this change is in the '%s' state, \
-it must be in the 'awaiting integration' state to begin integration",
-			cstate_state_ename(cstate_data->state)
-		);
-	}
+		change_fatal(cp, 0, i18n("bad ib state"));
+	if
+	(
+		!project_developer_may_integrate_get(pp)
+	&&
+		str_equal(change_developer_name(cp), user_name(up))
+	)
+		change_fatal(cp, 0, i18n("developer may not integrate"));
+	if
+	(
+		!project_reviewer_may_integrate_get(pp)
+	&&
+		str_equal(change_reviewer_name(cp), user_name(up))
+	)
+		change_fatal(cp, 0, i18n("reviewer may not integrate"));
 
 	/*
 	 * make sure only one integration at a time
 	 * for each project
 	 */
-	if (pstate_data->currently_integrating_change)
+	other = project_current_integration_get(pp);
+	if (other)
 	{
-		project_fatal
-		(
-			pp,
-"change %ld is the currently active integration, \
-only one integration may be performed at a time",
-			pstate_data->currently_integrating_change
-		);
+		sub_context_ty	*scp;
+
+		scp = sub_context_new();
+		sub_var_set_long(scp, "Number", magic_zero_decode(other));
+		project_fatal(pp, scp, i18n("currently integrating $number"));
+		/* NOTREACHED */
+		sub_context_delete(scp);
 	}
-	pstate_data->currently_integrating_change = change_number;
+	trace_long(change_number);
+	project_current_integration_set(pp, change_number);
 
 	/*
 	 * grab a delta number
 	 * and advance the project's delta counter
 	 */
-	cstate_data->delta_number = pstate_data->next_delta_number;
-	pstate_data->next_delta_number++;
+	cstate_data->delta_number = project_delta_number_get(pp);
 
 	/*
 	 * include the current year in the copyright_years field
 	 */
-	insert_this_year(pstate_data);
+	change_copyright_years_now(cp);
 
 	/*
 	 * Create the integration directory.
 	 */
+	base = str_format("delta%d", getpid());
+	num = str_format(".%3.3ld", cstate_data->delta_number);
+	os_become_orig();
+	base_max = os_pathconf_name_max(project_top_path_get(pp, 0));
+	os_become_undo();
+	base_max -= num->str_length;
+	if (base_max < 5)
+		base_max = 5;
 	s =
 		str_format
 		(
-			"%S/delta.%3.3ld",
-			project_home_path_get(pp),
-			cstate_data->delta_number
+			"%S/%.*S%S",
+			project_top_path_get(pp, 0),
+			base_max,
+			base,
+			num
 		);
+	str_free(base);
+	str_free(num);
 	change_integration_directory_set(cp, s);
 	str_free(s);
 
 	/*
-	 * There will be many files in the baseline in addition
-	 * to the sources files.
+	 * There will be many files in the baseline in addition to the
+	 * sources files.
 	 *
 	 * If any files are being deleted, only copy the source files
-	 * from the baseline to the integration directory.
-	 * This way the additional files relating to the removed sources file
-	 * are also removed (eg remove a .c file and you need to get rid of the
-	 * .o file).  It also shows when dependencies have become out-of-date.
+	 * from the baseline to the integration directory.  This way
+	 * the additional files relating to the removed sources file
+	 * are also removed (eg remove a .c file and you need to get
+	 * rid of the .o file).  It also shows when dependencies have
+	 * become out-of-date.
 	 *
 	 * It is possible to ask for this from the command line, too.
 	 */
-	for (j = 0; j < cstate_data->src->length; ++j)
+	if (maximum && minimum)
 	{
-		if (cstate_data->src->list[j]->action == file_action_remove)
+		mutually_exclusive_options
+		(
+			arglex_token_minimum,
+			arglex_token_maximum,
+			integrate_begin_usage
+		);
+	}
+	if (!maximum && !minimum)
+	{
+		for (j = 0; ; ++j)
 		{
-			minimum = 1;
-			break;
+			fstate_src	src_data;
+	
+			src_data = change_file_nth(cp, j);
+			if (!src_data)
+				break;
+			if (src_data->action == file_action_remove)
+			{
+				minimum = 1;
+				break;
+			}
 		}
 	}
 
@@ -768,52 +1050,79 @@ only one integration may be performed at a time",
 	dd = change_development_directory_get(cp, 1);
 	id = change_integration_directory_get(cp, 1);
 	bl = project_baseline_path_get(pp, 1);
-	project_become(pp);
 	errs = 0;
-	for (j = 0; j < cstate_data->src->length; ++j)
+	for (j = 0; ; ++j)
 	{
-		cstate_src	src_data;
+		fstate_src	src_data;
 		string_ty	*s1;
 		string_ty	*s2;
+		int		ok;
 
-		src_data = cstate_data->src->list[j];
+		src_data = change_file_nth(cp, j);
+		if (!src_data)
+			break;
 		if (src_data->usage == file_usage_build)
 			continue;
-		s1 = str_format("%S/%S", dd, src_data->file_name);
-		s2 = str_format("%S,D", s1);
-		switch (src_data->action)
-		{
-		case file_action_remove:
-			break;
+		if (src_data->action == file_action_remove)
+			continue;
 
-		case file_action_modify:
-		case file_action_create:
-			if
-			(
-				!os_exists(s1)
-			||
-				os_mtime(s1) != src_data->diff_time
-			)
-			{
-				change_error
-				(
-					cp,
-					"file \"%S\" has been altered",
-					s1
-				);
-				errs++;
-			}
-			break;
-		}
-		if (!os_exists(s2) || os_mtime(s2) != src_data->diff_file_time)
+		s1 = change_file_path(cp, src_data->file_name);
+		project_become(pp);
+		ok = change_fingerprint_same(src_data->file_fp, s1, 0);
+		project_become_undo();
+		if (!ok)
 		{
-			change_error(cp, "file \"%S\" has been altered", s2);
+			sub_context_ty	*scp;
+
+			scp = sub_context_new();
+			sub_var_set_string(scp, "File_Name", src_data->file_name);
+			change_error(cp, scp, i18n("$filename altered"));
+			sub_context_delete(scp);
 			errs++;
 		}
+		assert(src_data->file_fp->youngest > 0);
+		assert(src_data->file_fp->oldest > 0);
+
+		s2 = str_format("%S,D", s1);
+		project_become(pp);
+		ok = change_fingerprint_same(src_data->diff_file_fp, s2, 0);
+		project_become_undo();
+		if (!ok)
+		{
+			sub_context_ty	*scp;
+
+			scp = sub_context_new();
+			sub_var_set_format(scp, "File_Name", "%S,D", src_data->file_name);
+			change_error(cp, scp, i18n("$filename altered"));
+			sub_context_delete(scp);
+			errs++;
+		}
+		assert(src_data->diff_file_fp->youngest > 0);
+		assert(src_data->diff_file_fp->oldest > 0);
+		str_free(s1);
+		str_free(s2);
 	}
 	if (errs)
 		quit(1);
 
+	/*
+	 * add to the change history
+	 *	(The time stamp is used later for the file mod times.)
+	 */
+	os_throttle();
+	history_data = change_history_new(cp, up);
+	history_data->what = cstate_history_what_integrate_begin;
+
+	/*
+	 * Make sure the integration directory is not already there
+	 * (a user make have killed a previous aeib).  Register a rmdir
+	 * (to be done in the background) if an undo is needed.
+	 */
+	project_become(pp);
+	if (os_exists(id))
+		os_rmdir_tree(id);
+	undo_rmdir_bg(id);
+
 	/*
 	 * create the integration directory
 	 * copy everything from baseline to integration directory
@@ -822,7 +1131,7 @@ only one integration may be performed at a time",
 	 */
 	if (pconf_data->link_integration_directory)
 	{
-		change_verbose(cp, "link baseline to integration directory");
+		change_verbose(cp, 0, i18n("link baseline to integration directory"));
 		dir_walk
 		(
 			bl,
@@ -838,7 +1147,7 @@ only one integration may be performed at a time",
 	}
 	else
 	{
-		change_verbose(cp, "copy baseline to integration directory");
+		change_verbose(cp, 0, i18n("copy baseline to integration directory"));
 		dir_walk
 		(
 			bl,
@@ -856,14 +1165,16 @@ only one integration may be performed at a time",
 	/*
 	 * apply the changes to the integration directory
 	 */
-	change_verbose(cp, "apply change to integration directory");
-	for (j = 0; j < cstate_data->src->length; ++j)
+	change_verbose(cp, 0, i18n("apply change to integration directory"));
+	for (j = 0; ; ++j)
 	{
-		cstate_src	src_data;
+		fstate_src	src_data;
 		string_ty	*s1;
 		string_ty	*s2;
 
-		src_data = cstate_data->src->list[j];
+		src_data = change_file_nth(cp, j);
+		if (!src_data)
+			break;
 		s1 = str_format("%S/%S", dd, src_data->file_name);
 		s2 = str_format("%S/%S", id, src_data->file_name);
 		if (os_exists(s2))
@@ -876,6 +1187,14 @@ only one integration may be performed at a time",
 		}
 		switch (src_data->action)
 		{
+		case file_action_insulate:
+			/*
+			 * This should never happen: aede will fail if
+			 * there are any insulation files.
+			 */
+			assert(0);
+			break;
+
 		case file_action_remove:
 			break;
 
@@ -891,20 +1210,58 @@ only one integration may be performed at a time",
 			os_mkdir_between(id, src_data->file_name, 02755);
 			copy_whole_file(s1, s2, 0);
 			os_chmod(s2, 0444 & ~change_umask(cp));
+
+			/*
+			 * Make all of the change's files have the same
+			 * mod time, so that when aeipass flattens the
+			 * mod times, all of them fall into a single
+			 * second, minimizing the chance that mod times
+			 * will extend into the future after aeipass.
+			 *
+			 * This also helps cooperating DMTs flatten the
+			 * targets' mod times into a smaller range,
+			 * which also helps aeipass.
+			 */
+			os_mtime_set(s2, history_data->when);
 			break;
 		}
 		str_free(s1);
 		str_free(s2);
+
+		/*
+		 * clear the file's test times
+		 */
+		if (src_data->architecture_times)
+		{
+			fstate_src_architecture_times_list_type.free
+			(
+				src_data->architecture_times
+			);
+			src_data->architecture_times = 0;
+		}
+
+		/*
+		 * clear the file's integrate difference time
+		 */
+		if (src_data->idiff_file_fp)
+		{
+			fingerprint_type.free(src_data->idiff_file_fp);
+			src_data->idiff_file_fp = 0;
+		}
 	}
 	project_become_undo();
 
 	/*
 	 * add the change to the user's list
 	 */
+	trace_long(change_number);
 	user_own_add(up, project_name_get(pp), change_number);
 	cstate_data->state = cstate_state_being_integrated;
-	history_data = change_history_new(cp, up);
-	history_data->what = cstate_history_what_integrate_begin;
+
+	/*
+	 * clear the change build times,
+	 * and test times
+	 */
 	change_build_times_clear(cp);
 
 	/*
@@ -920,12 +1277,15 @@ only one integration may be performed at a time",
 	/*
 	 * run the integrate begin command
 	 */
+	pup = project_user(pp);
+	log_open(change_logfile_get(cp), pup, log_style);
+	user_free(pup);
 	change_run_integrate_begin_command(cp);
 
 	/*
 	 * verbose success message
 	 */
-	change_verbose(cp, "integration has begun");
+	change_verbose(cp, 0, i18n("integrate begin complete"));
 	change_free(cp);
 	project_free(pp);
 	user_free(up);
@@ -936,20 +1296,13 @@ only one integration may be performed at a time",
 void
 integrate_begin()
 {
-	trace(("integrate_begin()\n{\n"/*}*/));
-	switch (arglex())
+	static arglex_dispatch_ty dispatch[] =
 	{
-	default:
-		integrate_begin_main();
-		break;
+		{ arglex_token_help,		integrate_begin_help,	},
+		{ arglex_token_list,		integrate_begin_list,	},
+	};
 
-	case arglex_token_help:
-		integrate_begin_help();
-		break;
-
-	case arglex_token_list:
-		integrate_begin_list();
-		break;
-	}
-	trace((/*{*/"}\n"));
+	trace(("integrate_begin()\n{\n"));
+	arglex_dispatch(dispatch, SIZEOF(dispatch), integrate_begin_main);
+	trace(("}\n"));
 }
