@@ -1,6 +1,6 @@
 /*
  *	aegis - project change supervisor
- *	Copyright (C) 1991, 1992, 1993, 1994, 1995 Peter Miller;
+ *	Copyright (C) 1991-1999, 2001 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,31 +15,32 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: functions to impliment review fail
  */
 
-#include <stdio.h>
+#include <ac/stdio.h>
 #include <ac/stdlib.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#include <ael.h>
+#include <ael/change/by_state.h>
 #include <aerf.h>
 #include <arglex2.h>
-#include <change.h>
+#include <change/file.h>
 #include <commit.h>
-#include <dir.h>
 #include <error.h>
 #include <file.h>
 #include <help.h>
 #include <lock.h>
 #include <mem.h>
-#include <option.h>
 #include <os.h>
+#include <progname.h>
 #include <project.h>
+#include <project/file.h>
+#include <project_hist.h>
 #include <sub.h>
 #include <trace.h>
 #include <undo.h>
@@ -53,8 +54,9 @@ review_fail_usage()
 {
 	char		*progname;
 
-	progname = option_progname_get();
+	progname = progname_get();
 	fprintf(stderr, "usage: %s -Review_FAIL -File <reason-file> [ <option>... ]\n", progname);
+	fprintf(stderr, "       %s -Review_FAIL -REAson '<text>' [ <option>... ]\n", progname);
 	fprintf(stderr, "       %s -Review_FAIL -Edit [ <option>... ]\n", progname);
 	fprintf(stderr, "       %s -Review_FAIL -List [ <option>... ]\n", progname);
 	fprintf(stderr, "       %s -Review_FAIL -Help\n", progname);
@@ -67,20 +69,14 @@ static void review_fail_help _((void));
 static void
 review_fail_help()
 {
-	static char *text[] =
-	{
-#include <../man1/aerf.h>
-	};
-
-	help(text, SIZEOF(text), review_fail_usage);
+	help("aerfail", review_fail_usage);
 }
 
 
-static void review_fail_list _((void (*usage)(void)));
+static void review_fail_list _((void));
 
 static void
-review_fail_list(usage)
-	void		(*usage)_((void));
+review_fail_list()
 {
 	string_ty	*project_name;
 
@@ -92,17 +88,17 @@ review_fail_list(usage)
 		switch (arglex_token)
 		{
 		default:
-			generic_argument(usage);
+			generic_argument(review_fail_usage);
 			continue;
 
 		case arglex_token_project:
 			if (arglex() != arglex_token_string)
-				usage();
+				option_needs_name(arglex_token_project, review_fail_usage);
 			/* fall through... */
 
 		case arglex_token_string:
 			if (project_name)
-				fatal("duplicate -Project option");
+				duplicate_option_by_name(arglex_token_project, review_fail_usage);
 			project_name = str_from_c(arglex_value.alv_string);
 			break;
 		}
@@ -119,79 +115,6 @@ review_fail_list(usage)
 }
 
 
-static void repair_diff_time _((change_ty *, string_ty *));
-
-static void
-repair_diff_time(cp, path)
-	change_ty	*cp;
-	string_ty	*path;
-{
-	string_ty	*s;
-	cstate_src	src_data;
-	string_ty	*s2;
-
-	s = os_below_dir(change_development_directory_get(cp, 1), path);
-	src_data = change_src_find(cp, s);
-	if (src_data)
-		src_data->diff_time = os_mtime(path);
-	else
-	{
-		if
-		(
-			s->str_length > 2
-		&&
-			!strcmp(s->str_text + s->str_length - 2, ",D")
-		)
-		{
-			s2 = str_n_from_c(s->str_text, s->str_length - 2);
-			src_data = change_src_find(cp, s2);
-			if (src_data)
-				src_data->diff_file_time = os_mtime(path);
-			str_free(s2);
-		}
-	}
-	str_free(s);
-}
-
-
-static void rf_func _((void *, dir_walk_message_ty, string_ty *, struct stat *));
-
-static void
-rf_func(arg, message, path, st)
-	void		*arg;
-	dir_walk_message_ty message;
-	string_ty	*path;
-	struct stat	*st;
-{
-	change_ty	*cp;
-	int		uid;
-
-	trace(("rf_func(message = %d, path = %08lX, st = %08lX)\n{\n"/*}*/,
-		message, path, st));
-	cp = (change_ty *)arg;
-	trace_string(path->str_text);
-	switch (message)
-	{
-	case dir_walk_file:
-	case dir_walk_dir_before:
-		os_become_query(&uid, (int *)0, (int *)0);
-		if (st->st_uid == uid)
-		{
-			os_chmod(path, st->st_mode | 0200);
-			undo_chmod(path, st->st_mode);
-		}
-		repair_diff_time(cp, path);
-		break;
-
-	case dir_walk_dir_after:
-	case dir_walk_special:
-	case dir_walk_symlink:
-		break;
-	}
-	trace((/*{*/"}\n"));
-}
-
-
 static void check_permissions _((change_ty *, user_ty *));
 
 static void
@@ -201,46 +124,25 @@ check_permissions(cp, up)
 {
 	cstate		cstate_data;
 	project_ty	*pp;
-	pstate		pstate_data;
 
 	cstate_data = change_cstate_get(cp);
 	pp = cp->pp;
-	pstate_data = project_pstate_get(pp);
 
 	/*
 	 * it is an error if the change is not in the 'being_reviewed' state.
 	 */
 	if (cstate_data->state != cstate_state_being_reviewed)
-	{
-		change_fatal
-		(
-			cp,
-"this change is in the '%s' state, \
-it must be in the 'being reviewed' state to fail review",
-			cstate_state_ename(cstate_data->state)
-		);
-	}
+		change_fatal(cp, 0, i18n("bad rf state"));
 	if (!project_reviewer_query(pp, user_name(up)))
-	{
-		project_fatal
-		(
-			pp,
-			"user \"%S\" is not a reviewer",
-			user_name(up)
-		);
-	}
+		project_fatal(pp, 0, i18n("not a reviewer"));
 	if
 	(
-		!pstate_data->developer_may_review
+		!project_developer_may_review_get(pp)
 	&&
 		str_equal(change_developer_name(cp), user_name(up))
 	)
 	{
-		change_fatal
-		(
-			cp,
-			"the developer of a change may not also review it"
-		);
+		change_fatal(cp, 0, i18n("developer may not review"));
 	}
 }
 
@@ -250,11 +152,12 @@ static void review_fail_main _((void));
 static void
 review_fail_main()
 {
-	string_ty	*dd;
+	string_ty	*s;
+	sub_context_ty	*scp;
 	cstate		cstate_data;
-	pstate		pstate_data;
 	cstate_history	history_data;
 	string_ty	*comment = 0;
+	char		*reason = 0;
 	int		j;
 	string_ty	*project_name;
 	project_ty	*pp;
@@ -262,12 +165,13 @@ review_fail_main()
 	change_ty	*cp;
 	user_ty		*up;
 	user_ty		*devup;
-	int		edit;
+	edit_ty		edit;
 
 	trace(("review_fail_main()\n{\n"/*}*/));
+	arglex();
 	project_name = 0;
 	change_number = 0;
-	edit = 0;
+	edit = edit_not_set;
 	while (arglex_token != arglex_token_eoln)
 	{
 		switch (arglex_token)
@@ -277,75 +181,147 @@ review_fail_main()
 			continue;
 
 		case arglex_token_string:
-			error
-			(
-"warning: please use the -File option when specifying a reason file, \
-the unadorned form is now obsolescent"
-			);
+			scp = sub_context_new();
+			sub_var_set_charstar(scp, "Name", arglex_token_name(arglex_token_file));
+			error_intl(scp, i18n("warning: use $name option"));
+			sub_context_delete(scp);
 			if (comment)
-				fatal("too many files named");
+				fatal_too_many_files();
 			goto read_input_file;
 
 		case arglex_token_file:
 			if (comment)
-				goto duplicate;
-			if (arglex() != arglex_token_string)
+				duplicate_option(review_fail_usage);
+			switch (arglex())
 			{
-				error
+			default:
+				option_needs_file
 				(
-				 "the -File option requires a filename argument"
+					arglex_token_file,
+					review_fail_usage
 				);
-				review_fail_usage();
+				/*NOTREACHED*/
+
+			case arglex_token_string:
+				read_input_file:
+				os_become_orig();
+				s = str_from_c(arglex_value.alv_string);
+				comment = read_whole_file(s);
+				str_free(s);
+				os_become_undo();
+				break;
+
+			case arglex_token_stdio:
+				os_become_orig();
+				comment = read_whole_file((string_ty *)0);
+				os_become_undo();
+				break;
 			}
-			read_input_file:
-			os_become_orig();
-			comment = read_whole_file(arglex_value.alv_string);
-			os_become_undo();
+			assert(comment);
+			break;
+
+		case arglex_token_reason:
+			if (reason)
+				duplicate_option(review_fail_usage);
+			if (arglex() != arglex_token_string)
+				option_needs_string(arglex_token_reason, review_fail_usage);
+			reason = arglex_value.alv_string;
 			break;
 
 		case arglex_token_change:
 			if (arglex() != arglex_token_number)
-				review_fail_usage();
+				option_needs_number(arglex_token_change, review_fail_usage);
 			/* fall through... */
 
 		case arglex_token_number:
 			if (change_number)
-				fatal("duplicate -Change option");
+				duplicate_option_by_name(arglex_token_change, review_fail_usage);
 			change_number = arglex_value.alv_number;
-			if (change_number < 1)
-				fatal("change %ld out of range", change_number);
+			if (change_number == 0)
+				change_number = MAGIC_ZERO;
+			else if (change_number < 1)
+			{
+				scp = sub_context_new();
+				sub_var_set_long(scp, "Number", change_number);
+				fatal_intl(scp, i18n("change $number out of range"));
+				sub_context_delete(scp);
+			}
 			break;
 
 		case arglex_token_project:
 			if (project_name)
-				goto duplicate;
+				duplicate_option(review_fail_usage);
 			if (arglex() != arglex_token_string)
-				review_fail_usage();
+				option_needs_name(arglex_token_project, review_fail_usage);
 			project_name = str_from_c(arglex_value.alv_string);
 			break;
 
 		case arglex_token_edit:
-			if (edit)
+			if (edit == edit_foreground)
+				duplicate_option(review_fail_usage);
+			if (edit != edit_not_set)
 			{
-				duplicate:
-				fatal
+				too_many_edits:
+				mutually_exclusive_options
 				(
-					"duplicate %s option",
-					arglex_value.alv_string
+					arglex_token_edit,
+					arglex_token_edit_bg,
+					review_fail_usage
 				);
 			}
-			edit++;
+			edit = edit_foreground;
+			break;
+
+		case arglex_token_edit_bg:
+			if (edit == edit_background)
+				duplicate_option(review_fail_usage);
+			if (edit != edit_not_set)
+				goto too_many_edits;
+			edit = edit_background;
+			break;
+
+		case arglex_token_wait:
+		case arglex_token_wait_not:
+			user_lock_wait_argument(review_fail_usage);
 			break;
 		}
 		arglex();
 	}
-	if (edit && comment)
-		fatal("may not use both -Edit and -File options");
-	if (!edit && !comment)
+	if (comment && reason)
 	{
-		error("warning: no -File specified, assuming -Edit desired");
-		++edit;
+		mutually_exclusive_options
+		(
+			arglex_token_file,
+			arglex_token_reason,
+			review_fail_usage
+		);
 	}
+	if (edit != edit_not_set && (comment || reason))
+	{
+		mutually_exclusive_options
+		(
+			(
+				edit == edit_foreground
+			?
+				arglex_token_edit
+			:
+				arglex_token_edit_bg
+			),
+			(comment ? arglex_token_file : arglex_token_reason),
+			review_fail_usage
+		);
+	}
+	if (edit == edit_not_set && !(comment || reason))
+	{
+		scp = sub_context_new();
+		sub_var_set_charstar(scp, "Name1", arglex_token_name(arglex_token_file));
+		sub_var_set_charstar(scp, "Name2", arglex_token_name(arglex_token_edit));
+		error_intl(scp, i18n("warning: no $name1, assuming $name2"));
+		sub_context_delete(scp);
+		edit = edit_foreground;
+	}
+	if (reason)
+		comment = str_from_c(reason);
 
 	/*
 	 * locate project data
@@ -373,10 +349,10 @@ the unadorned form is now obsolescent"
 	 * create the commant, if necessary
 	 * check permissions first
 	 */
-	if (edit)
+	if (edit != edit_not_set)
 	{
 		check_permissions(cp, up);
-		comment = os_edit_new();
+		comment = os_edit_new(edit);
 	}
 
 	/*
@@ -384,10 +360,9 @@ the unadorned form is now obsolescent"
 	 */
 	project_pstate_lock_prepare(pp);
 	change_cstate_lock_prepare(cp);
-	lock_prepare_ustate_all(); /* we don't know which user until later */
+	lock_prepare_ustate_all(0, 0); /* we don't know which user until later */
 	lock_take();
 	cstate_data = change_cstate_get(cp);
-	pstate_data = project_pstate_get(pp);
 
 	/*
 	 * check they are allowed to
@@ -404,7 +379,6 @@ the unadorned form is now obsolescent"
 	history_data = change_history_new(cp, up);
 	history_data->what = cstate_history_what_review_fail;
 	history_data->why = comment;
-	change_build_times_clear(cp);
 
 	/*
 	 * add it back into the user's change list
@@ -412,26 +386,19 @@ the unadorned form is now obsolescent"
 	devup = user_symbolic(pp, change_developer_name(cp));
 	user_own_add(devup, project_name_get(pp), change_number);
 
-	/*
-	 * Restore write permission to the change files
-	 * and repair the diff time fields.
-	 */
-	dd = change_development_directory_get(cp, 1);
-	user_become(devup);
-	dir_walk(dd, rf_func, cp);
-	os_become_undo();
-
 	/*
 	 * go through the files in the change and unlock them
 	 * in the baseline
 	 */
-	for (j = 0; j < cstate_data->src->length; ++j)
+	for (j = 0; ; ++j)
 	{
-		cstate_src	c_src_data;
-		pstate_src	p_src_data;
+		fstate_src	c_src_data;
+		fstate_src	p_src_data;
 
-		c_src_data = cstate_data->src->list[j];
-		p_src_data = project_src_find(pp, c_src_data->file_name);
+		c_src_data = change_file_nth(cp, j);
+		if (!c_src_data)
+			break;
+		p_src_data = project_file_find(pp, c_src_data->file_name);
 		if (!p_src_data)
 			continue;
 		p_src_data->locked_by = 0;
@@ -440,13 +407,22 @@ the unadorned form is now obsolescent"
 		 * Remove the file if it is about_to_be_created
 		 * by the change we are rescinding.
 		 */
-		if (p_src_data->about_to_be_created_by)
+		if
+		(
+			p_src_data->about_to_be_created_by
+		||
+			p_src_data->about_to_be_copied_by
+		)
 		{
-			assert(p_src_data->about_to_be_created_by == change_number);
+			assert(!p_src_data->about_to_be_created_by || p_src_data->about_to_be_created_by == change_number);
+			assert(!p_src_data->about_to_be_copied_by || p_src_data->about_to_be_copied_by == change_number);
 			if (p_src_data->deleted_by)
+			{
+				assert(!p_src_data->about_to_be_copied_by);
 				p_src_data->about_to_be_created_by = 0;
+			}
 			else
-				project_src_remove(pp, c_src_data->file_name);
+				project_file_remove(pp, c_src_data->file_name);
 		}
 	}
 
@@ -467,7 +443,7 @@ the unadorned form is now obsolescent"
 	/*
 	 * verbose success message
 	 */
-	change_verbose(cp, "failed review, returned to developer");
+	change_verbose(cp, 0, i18n("review fail complete"));
 	change_free(cp);
 	project_free(pp);
 	user_free(up);
@@ -479,20 +455,13 @@ the unadorned form is now obsolescent"
 void
 review_fail()
 {
-	trace(("review_fail()\n{\n"/*}*/));
-	switch (arglex())
+	static arglex_dispatch_ty dispatch[] =
 	{
-	default:
-		review_fail_main();
-		break;
+		{ arglex_token_help,		review_fail_help,	},
+		{ arglex_token_list,		review_fail_list,	},
+	};
 
-	case arglex_token_help:
-		review_fail_help();
-		break;
-
-	case arglex_token_list:
-		review_fail_list(review_fail_usage);
-		break;
-	}
-	trace((/*{*/"}\n"));
+	trace(("review_fail()\n{\n"));
+	arglex_dispatch(dispatch, SIZEOF(dispatch), review_fail_main);
+	trace(("}\n"));
 }
