@@ -1,6 +1,6 @@
 /*
  *	aegis - project change supervisor
- *	Copyright (C) 1991, 1992, 1993, 1994 Peter Miller;
+ *	Copyright (C) 1991, 1992, 1993, 1994, 1995, 1997, 1998, 1999, 2001 Peter Miller;
  *	All rights reserved.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -15,18 +15,19 @@
  *
  *	You should have received a copy of the GNU General Public License
  *	along with this program; if not, write to the Free Software
- *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
  *
  * MANIFEST: lexical analysis of command line arguments
  */
 
 #include <ac/stddef.h>
 #include <ac/string.h>
-#include <ctype.h>
+#include <ac/ctype.h>
 
 #include <arglex.h>
 #include <error.h>
-#include <option.h>
+#include <mem.h>
+#include <progname.h>
 #include <str.h>
 #include <trace.h>
 
@@ -35,15 +36,16 @@ static arglex_table_ty table[] =
 	{ "-",			arglex_token_stdio,		},
 	{ "-Help",		arglex_token_help,		},
 	{ "-VERSion",		arglex_token_version,		},
-	{ "-TRace",		arglex_token_trace,		},
+	{ "-TRAce",		arglex_token_trace,		},
 };
 
 static	int		argc;
 static	char		**argv;
 	arglex_value_ty	arglex_value;
-	arglex_token_ty	arglex_token;
+	int		arglex_token;
 static	arglex_table_ty	*utable;
-static	char		*partial;
+static	const char	*partial;
+static	int		strings_only_mode;
 
 
 /*
@@ -72,7 +74,7 @@ arglex_init(ac, av, tp)
 	char		**av;
 	arglex_table_ty	*tp;
 {
-	option_progname_set(av[0]);
+	progname_set(av[0]);
 	argc = ac - 1;
 	argv = av + 1;
 	utable = tp;
@@ -130,12 +132,12 @@ arglex_init(ac, av, tp)
 
 int
 arglex_compare(formal, actual)
-	char	*formal;
-	char	*actual;
+	const char	*formal;
+	const char	*actual;
 {
-	char	fc;
-	char	ac;
-	int	result;
+	char		fc;
+	char		ac;
+	int		result;
 
 	trace(("arglex_compare(formal = \"%s\", actual = \"%s\")\n{\n",
 		formal, actual));
@@ -387,7 +389,7 @@ is_a_number(s)
  *	Must call arglex_init befor this function is called.
  */
 
-arglex_token_ty
+int
 arglex()
 {
 	arglex_table_ty	*tp;
@@ -410,6 +412,7 @@ arglex()
 	}
 	else
 	{
+		get_another:
 		if (argc <= 0)
 		{
 			arglex_token = arglex_token_eoln;
@@ -420,6 +423,21 @@ arglex()
 		argc--;
 		argv++;
 
+		/*
+		 * The ``--'' option means the rest of the arguments on
+		 * the command line are only strings.
+		 */
+		if (strings_only_mode)
+		{
+			arglex_token = arglex_token_string;
+			goto done;
+		}
+		if (arg[0] == '-' && arg[1] == '-' && !arg[2])
+		{
+			strings_only_mode = 1;
+			goto get_another;
+		}
+
 		/*
 		 * See if it looks like a GNU "-foo=bar" option.
 		 * Split it at the '=' to make it something the
@@ -501,7 +519,7 @@ arglex()
 		one:
 		arglex_token = hit[0]->t_token;
 		if (partial)
-			arg = partial;
+			arg = (char *)partial; /* const-ness hack */
 		else
 			arg = hit[0]->t_name;
 		break;
@@ -527,11 +545,11 @@ arglex()
 		{
 			s2 = str_format("%S, %s", s1, hit[j]->t_name);
 			str_free(s1);
-			s2 = s2;
+			s1 = s2;
 		}
-		fatal
+		fatal_raw
 		(
-			"option \"%s\" abmiguous (%s)",
+			"option \"%s\" ambiguous (%s)",
 			arg,
 			s1->str_text
 		);
@@ -543,6 +561,239 @@ arglex()
 	done:
 	arglex_value.alv_string = arg;
 	trace(("return %d; /* %s */\n", arglex_token, arglex_value.alv_string));
-	trace((/*{*/"}\n"));
+	trace(("}\n"));
 	return arglex_token;
 }
+
+
+/*
+ * NAME
+ *	arglex_prefetch
+ *
+ * SYNOPSIS
+ *	arglex_token_ty arglex_prefetch(arglex_token_ty *list, int list_len);
+ *
+ * DESCRIPTION
+ *	The arglex_prefetch function is used to perfom lexical analysis
+ *	on the command line arguments, much like the arglex function.
+ *	However, it is given a list of token to look for on the command
+ *	line, and such arguments are matched and extracted, which may
+ *	be used to relax command line argument ordering restrictions.
+ *
+ * RETURNS
+ *	One of the tokens in the list, or
+ *	ARGLEX_PREFETCH_FAIL if none of the are available.
+ *
+ * CAVEAT
+ *	Must call arglex_init before this function is called.
+ */
+
+int
+arglex_prefetch(list, list_len)
+	arglex_token_ty	*list;
+	int		list_len;
+{
+	int		j;
+
+	trace(("arglex_prefetch()\n{\n"));
+	if (strings_only_mode)
+		goto fail;
+
+	for (j = 0; j < argc; ++j)
+	{
+		char		*actual;
+		int		k;
+
+		/*
+		 * The ``--'' option means the rest of the arguments on
+		 * the command line are only strings.
+		 */
+		actual = argv[j];
+		if (actual[0] == '-' && actual[1] == '-' && !actual[2])
+			goto fail;
+	
+		/*
+		 * see if it is a number
+		 */
+		if (is_a_number(actual))
+			continue;
+	
+		/*
+		 * Turn the GNU-style leading "--"
+		 * into "-" if necessary.
+		 */
+		if
+		(
+			actual[0] == '-'
+		&&
+			actual[1] == '-'
+		&&
+			actual[2]
+		&&
+			!is_a_number(actual + 1)
+		)
+			++actual;
+
+		for (k = 0; k < list_len; ++k)
+		{
+			int token = list[k];
+			char *formal = arglex_token_name(token);
+			if (arglex_compare(formal, actual))
+			{
+				int		m;
+
+				/*
+				 * Shuffle everything down to fill in
+				 * the hole.
+				 */
+				for (m = j + 1; m < argc; ++m)
+					argv[m - 1] = argv[m];
+				--argc;
+
+				/*
+				 * Fill in the answer as it would be
+				 * returned form arglex()
+				 */
+				arglex_value.alv_string = actual;
+				arglex_token = token;
+				trace(("return %d; /* %s */\n", arglex_token,
+					arglex_value.alv_string));
+				trace(("}\n"));
+				return arglex_token;
+			}
+		}
+	}
+
+	fail:
+	trace(("return FAIL;\n"));
+	trace(("}\n"));
+	return ARGLEX_PREFETCH_FAIL;
+}
+
+
+char *
+arglex_token_name(n)
+	arglex_token_ty	n;
+{
+	arglex_table_ty	*tp;
+
+	switch (n)
+	{
+	case arglex_token_eoln:
+		return "end of command line";
+
+	case arglex_token_number:
+		return "number";
+
+	case arglex_token_option:
+		return "option";
+
+	case arglex_token_stdio:
+		return "standard input or output";
+
+	case arglex_token_string:
+		return "string";
+
+	default:
+		break;
+	}
+	for (tp = table; tp < ENDOF(table); tp++)
+	{
+		if (tp->t_token == n)
+			return tp->t_name;
+	}
+	if (utable)
+	{
+		for (tp = utable; tp->t_name; tp++)
+		{
+			if (tp->t_token == n)
+				return tp->t_name;
+		}
+	}
+
+	assert(0);
+	return "unknown command line token";
+}
+
+
+arglex_table_ty *
+arglex_table_catenate(tp1, tp2)
+	arglex_table_ty	*tp1;
+	arglex_table_ty	*tp2;
+{
+	size_t		len1;
+	size_t		len2;
+	size_t		len;
+	arglex_table_ty	*tp;
+	static arglex_table_ty zero = ARGLEX_END_MARKER;
+
+	for (len1 = 0; tp1[len1].t_name; ++len1)
+		;
+	for (len2 = 0; tp2[len2].t_name; ++len2)
+		;
+	len = len1 + len2;
+	tp = mem_alloc((len + 1) * sizeof(arglex_table_ty));
+	memcpy(tp, tp1, len1 * sizeof(arglex_table_ty));
+	memcpy(tp + len1, tp2, len2 * sizeof(arglex_table_ty));
+	tp[len] = zero;
+	return tp;
+}
+
+
+/*
+ * NAME
+ *	arglex_dispatch
+ *
+ * DESCRIPTION
+ *	The arglex_dispatch function is used to dispatch into a function
+ *	table, based on the presence of one of a given list of command
+ *	line arguments being poresent on the command line.
+ *
+ *	If none of them are present, the `the_default' argument (if non-zero)
+ *	is a default function to be called.
+ */
+
+void
+arglex_dispatch(choices, choices_len, the_default)
+	arglex_dispatch_ty *choices;
+	int		choices_len;
+	void		(*the_default)_((void));
+{
+	int		j;
+	int		*tmp;
+	int		tmp_len;
+	int		tok;
+	int		priority;
+
+	trace(("arglex_dispatch()\n{\n"));
+	tmp = mem_alloc(choices_len * sizeof(int));
+	for (priority = 0; ; ++priority)
+	{
+		tmp_len = 0;
+		for (j = 0; j < choices_len; ++j)
+		{
+			arglex_dispatch_ty *cp = choices + j;
+			if (cp->priority == priority)
+				tmp[tmp_len++] = cp->token;
+		}
+		if (tmp_len == 0)
+			break;
+		tok = arglex_prefetch(tmp, tmp_len);
+		for (j = 0; j < choices_len; ++j)
+		{
+			arglex_dispatch_ty *cp = choices + j;
+			if (tok == cp->token)
+			{
+				mem_free(tmp);
+				if (cp->func)
+					cp->func();
+				trace(("}\n"));
+				return;
+			}
+		}
+	}
+	mem_free(tmp);
+	if (the_default)
+		the_default();
+	trace(("}\n"));
+}
