@@ -1,7 +1,7 @@
 #!/bin/sh
 #
 #	aegis - project change supervisor
-#	Copyright (C) 1994, 1995 Peter Miller;
+#	Copyright (C) 1994, 1995, 1999, 2000, 2001 Peter Miller;
 #	All rights reserved.
 #
 #	This program is free software; you can redistribute it and/or modify
@@ -16,12 +16,12 @@
 #
 #	You should have received a copy of the GNU General Public License
 #	along with this program; if not, write to the Free Software
-#	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 #
 # MANIFEST: shell script to notify 'aedb -User' events
 #
 # Suggested project attribute:
-# forced_develop_begin_notify_command = "$lib/db_forced.sh $p $c $developer";
+# forced_develop_begin_notify_command = "$datadir/db_forced.sh $p $c $developer";
 #
 aegis=aegis
 case $# in
@@ -57,13 +57,14 @@ if [ $? -ne 0 ]; then quit; fi
 #
 # include full details of the change
 #
-$aegis -list change_details -project $project -change $change -verbose >> $tmp
+$aegis -list change_details -project $project -change $change -verbose \
+	-pl=66 -pw=68 >> $tmp
 if [ $? -ne 0 ]; then quit; fi
 
 #
 # mail it to all reviewers
 #
-mail $developer < $tmp
+mail -s "Project $project: Change $change: Develop Begin" $developer < $tmp
 if [ $? -ne 0 ]; then quit; fi
 
 #
