@@ -1,7 +1,7 @@
 #!/bin/sh
 #
 #	aegis - project change supervisor
-#	Copyright (C) 1992, 1993, 1995 Peter Miller;
+#	Copyright (C) 1992, 1993, 1995, 1999, 2000, 2001 Peter Miller;
 #	All rights reserved.
 #
 #	This program is free software; you can redistribute it and/or modify
@@ -16,12 +16,12 @@
 #
 #	You should have received a copy of the GNU General Public License
 #	along with this program; if not, write to the Free Software
-#	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
 #
 # MANIFEST: command used to notify by email that a change has passed review
 #
 # Suggested project attribute:
-# review_pass_notify_command = "$lib/rp.sh $p $c $developer $reviewer";
+# review_pass_notify_command = "$datadir/rp.sh $p $c $developer $reviewer";
 #
 aegis=aegis
 case $# in
@@ -70,13 +70,14 @@ if [ $? -ne 0 ]; then quit; fi
 #
 # include full details of the change
 #
-$aegis -list change_details -project $project -change $change -verbose >> $tmp
+$aegis -list change_details -project $project -change $change -verbose \
+	-pl=66 -pw=80 >> $tmp
 if [ $? -ne 0 ]; then quit; fi
 
 #
 # mail it to the developer and all integrators
 #
-mail $developer $integrators < $tmp
+mail -s "Project $project: Change $change: passed review" $developer $integrators < $tmp
 if [ $? -ne 0 ]; then quit; fi
 
 #
